<?php

/**
 * @file
 * sugarondrupal.admin.inc
 *
 * provides administrative pages
 *
 */

// configuration form
function sugarondrupal_admin_settings() {
  $form = array();

  $form['sugarondrupal_webservice'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('SugarCRM Web Service End Point'),
    '#description'    => t('The End Point is the Web Services handler you would like to use from your SugarCRM setup. You may select SOAP or REST end points.'),
  );
  $form['sugarondrupal_webservice']['sugarondrupal_webservice_endpoint'] = array(
    '#type'           => 'textfield',
    '#title'          => t('SugarCRM End Point'),
    '#size'           => 120,
    '#maxlength'      => 256,
    '#default_value'  => variable_get('sugarondrupal_webservice_endpoint', NULL),
    '#description'    => t('Enter the URL of your SugarCRM web service. Valid examples are: http://server/sugarcrm/service/v2/rest.php, http://server/service/v2/soap.php?wsdl.'),
  );

  // Do not worry about SOAP for now.
  $protocols = array('rest' => 'REST protocol');
  
  $form['sugarondrupal_webservice']['sugarondrupal_webservice_protocol'] = array(
    '#type'           => 'radios',
    '#title'          => t('SugarCRM Web Service Protocol'),
    '#options'        => $protocols,
    '#default_value'  => variable_get('sugarondrupal_webservice_protocol', 'rest'),
    '#description'    => t('Choose if the Web Service End Point is a SOAP or RESTful handler. REST services are only available from SugarCRM versions >= 5.5'),
  );

  // When php < 5.2 or JSON is not enabled in PHP, we provide our custom compat
  // json encoder.
  module_load_include('php', 'sugarondrupal', 'compat_json');
  $encodings = array(
    'JSON'      => t('JSON encoding'),
    'SERIALIZE' => t('PHP serialization'),
  );

  $form['sugarondrupal_webservice']['sugarondrupal_rest_encoding'] = array(
    '#type'           => 'radios',
    '#title'          => t('SugarCRM RESTful encoding'),
    '#options'        => $encodings,
    '#default_value'  => variable_get('sugarondrupal_rest_encoding', 'JSON'),
    '#description'    => t('If the Web Service End Point is a RESTful service, select the type of encoding for the communication. <strong>versions of <= SugarCRM 5.5.BETA1 should use JSON enconding</strong>.'),
  );

  $form['sugarondrupal_webservice']['sugarondrupal_debug'] = array(
    '#type'           => 'checkbox',
    '#title'          => t('Debug information'),
    '#default_value'  => variable_get('sugarondrupal_debug', FALSE),
    '#description'    => t('Record debug information using Watchdog. Debug information can affect the site performance. Enable this option only if you are experiencing errors or unexpected behavior.'),
  );

  // SugarCRM Authentication
  $form['sugarondrupal_authentication'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('SugarCRM Authentication'),
    '#collapsible'    => variable_get('sugarondrupal_password', ''),
    '#collapsed'      => variable_get('sugarondrupal_password', ''),
    '#description'    => t('The user and password of these fields should have access to some of the modules and information of your SugarCRM setup. Read carefully the documentation about how to prepare a secure user for this integration.'),
  );
  $form['sugarondrupal_authentication']['sugarondrupal_user'] = array(
    '#type'           => 'textfield',
    '#title'          => t('User login'),
    '#size'           => 32,
    '#maxlength'      => 64,
    '#default_value'  => variable_get('sugarondrupal_user', ''),
    '#description'    => t('Enter the login name of the account used to connect Drupal with the SugarCRM installation'),
  );
  $form['sugarondrupal_authentication']['sugarondrupal_password'] = array(
    '#type'           => 'password',
    '#title'          => t('Password'),
    '#size'           => 32,
    '#maxlength'      => 64,
    '#default_value'  => variable_get('sugarondrupal_password', ''),
    '#description'    => t('Enter the password of the account'),
  );

  $form['buttons']['update_session_id'] = array(
    '#type'   => 'submit',
    '#value'  => 'Update authenticated session',
    '#submit' => array('sugarondrupal_get_valid_session'),
    '#weight' => 10,
  );

  return system_settings_form($form);
}


/*
 * Validator hook for settings. Check that that end point is set.
 *
 * @todo: remove the SOAP endpoint check when SOAP is supported.
 */
function sugarondrupal_admin_settings_validate($form, &$form_state) {
  // There's no way to continue without a base url.
  if ($form_state['values']['sugarondrupal_webservice_endpoint'] == '') {
    form_set_error('sugarondrupal_webservice_endpoint', t('The Web Service End point is required.'));
  }

  // SOAP is not supported for now.
  if (!strtok('?wsdl', $form_state['values']['sugarondrupal_webservice_endpoint'])) {
    form_set_error('sugarondrupal_webservice_endpoint', t('SOAP Service End point is not yet supported, please configure the module to use a RESTful end point.'));
  }

  // Keep current password set on form submission when password field is empty.
  if ($form_state['values']['sugarondrupal_password'] == '' && variable_get('sugarondrupal_password', '') != '') {
    $form_state['values']['sugarondrupal_password'] = variable_get('sugarondrupal_password', '');
  }
}

/*
 * Get return current session or create a new one
 */
function sugarondrupal_get_valid_session($form, $form_state) {
  //delete current session
  variable_set('sugarondrupal_session', NULL);
  $session = sugarondrupal_session_get();
  if ($session) {
    drupal_set_message(t('Got a valid session id. Both applications are now connected.'));
  }
  else {
    drupal_set_message(t('Unable to get a valid session, check username and password.'), 'error');
  }
}

/**
 * Integration status information
 */

/**
 * SugarCRM and Drupal integration status page.
 *
 * Show information about SugarCRM and Drupal integration status to the site
 * administrator. The page is created as a form to allow other extension modules
 * to include additional information.
 *
 * @return
 *   A form showing current integration status.
 */
function sugarondrupal_admin_status() {
  $form = array();

  // verify that current session is still working before continuing.
  if (!sugarondrupal_get_user_id()) {
    drupal_set_message(t('The session used to communicate Drupal with SugarCRM is invalid. Configure your SugarCRM connection or authenticate the session again in the !url page.', array('!url' => l("SugarCRM connection settings", 'admin/settings/sugarondrupal'), '!uninstall' => l("uninstall the module", 'admin/build/modules'))), 'error');
    return $form;
  }

  /**
   * Generic SugarCRM server information.
   */
  $form['sugarondrupal_server'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('SugarCRM information'),
  );
  $rows = array();

  $info = sugarondrupal_get_server_info();
  $rows[] = array(t('SugarCRM version'), sugarondrupal_sugarcrm_version_string($info));
  if (isset($info['gmt_time'])) {
    $rows[] = array(t('GMT time'), $info['gmt_time']);
  }

  $form['sugarondrupal_server']['sugarondrupal_server_info'] = array(
    '#type'           => 'markup',
    '#value'          => theme('table', array(), $rows),
  );

  /**
   * SugarCRM connection information.
   */
  $form['sugarondrupal_conn'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('SugarCRM Connection'),
    '#description'    => t('The information shown below is related to the account used to initialize the connection bewteen Drupal and SugarCRM.')
  );
  $rows = array();

  $info = variable_get('sugarondrupal_login', array());
  $values = _sugarondrupal_name_value_list_to_array($info['name_value_list']);
  
  if (count($values)) {
    // Deliberately hide the session Id.
    // $rows[] = array(t('Session id'), isset($info['id']) ? $info['id'] : '');
    $rows[] = array(t('Connected using entry'), _sugarondrupal_l($values['user_name'], "index.php?module=" . $info['module_name'] . "&action=DetailView&record=" . $values['user_id']));
    $rows[] = array(t('Language'), $values['user_language']);
    $rows[] = array(t('Currency'), $values['user_currency_name']);
  }
  else {
    $rows[] = array(t('No connection information found.'));
  }

  $form['sugarondrupal_conn']['sugarondrupal_conn_info'] = array(
    '#type'           => 'markup',
    '#value'          => theme('table', array(), $rows),
  );

  /**
   * Module information
   */
  $form['sugarondrupal_mod'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('SugarCRM Modules'),
    '#description'    => t('List of available modules.'),
  );
  $rows = array();

  $modules = sugarondrupal_get_available_modules();
  foreach ($modules as $id => $name) {
    $rows[] = array(l($name, 'admin/reports/status/sugarcrm/' . $name));
  }
  $form['sugarondrupal_mod']['sugarondrupal_modules'] = array(
    '#type'           => 'markup',
    '#value'          => theme('table', array(), $rows),
  );

  /**
   * Dummy placeholder
   */
  $form['sugarondrupal_placeholder'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('SugarCRM information placeholder'),
    '#description'    => t('Dummy status information TODO: Improve me.'),
  );

  return $form;
}

/**
 * Build a link to SugarCRM.
 *
 * Very alpha code, need some improvements or create a specific sugarondrupal
 * variable to hold the SugarCRM url.
 */
function _sugarondrupal_l($text, $path, $options = array()) {
  if (!$url = variable_get('sugarondrupal_webservice_endpoint', NULL)) {
    return NULL;
  }

  // Parse service url removing any service reference.
  $data = preg_split('/service\/v2/i', $url);
  $path = $data[0] . $path;

  return l($text, $path, $options);
}

/**
 * SugarCRM module information status page.
 *
 * Show information about...
 *
 * @param $name
 *   Module name to show information about.
 *
 * @return
 *   A form showing current integration status.
 */
function sugarondrupal_admin_module_status($form_state, $name = NULL) {
  $form = array();

  // Go back to SugarCRM status information page if no Module is specified.
  if (!isset($name)) {
    drupal_goto('admin/reports/status/sugarcrm');
  }

  // Get all information available for this module.
  $params = array(
    'module_name' => $name,
    'fields'      => array(),
  );
  $info = sugarondrupal_get_module_fields($params);

  if (isset($info['number']) && $info['number'] == 20) {
    drupal_set_message(t('!message', array('!message' => isset($info['description']) ? $info['description'] : 'Module not available')), 'error');
    return $form;
  }

  /**
   * Module fields.
   */
  $form['sugarondrupal_fields'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('SugarCRM Module fields'),
    '#description'    => t('List of available fields for module: !module.', array('!module' => $info['module_name'])),
  );
  $rows = array();

  foreach ($info['module_fields'] as $id => $field) {
    $rows[] = array(
      isset($field['label']) ? $field['label'] : '',
      isset($field['name']) ? $field['name'] : '',
      isset($field['type']) ? $field['type'] : '',
      isset($field['required']) ? $field['required'] : '',
    );
  }
  $header = array(
    t('Label'),
    t('Name'),
    t('Type'),
    t('Required'),
  );
  $form['sugarondrupal_fields']['sugarondrupal_module_fields'] = array(
    '#type'           => 'markup',
    '#value'          => theme('table', $header, $rows),
  );

  /**
   * Module relations.
   */
  $form['sugarondrupal_rela'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('SugarCRM Module Relations'),
    '#description'    => t('List of available relations for module: !module.', array('!module' => $info['module_name'])),
  );
  $rows = array();

  foreach ($info['link_fields'] as $id => $link) {
    $rows[] = array(
      isset($link['name']) ? $link['name'] : '',
      isset($link['module']) ? $link['module'] : '',
      isset($link['type']) ? $link['type'] : '',
      isset($link['relationship']) ? $link['relationship'] : '',
      isset($link['bean_name'])? $link['bean_name'] : '',
    );
  }
  $header = array(
    t('Name'),
    t('Module'),
    t('Type'),
    t('Relationship'),
    t('Bean Name'),
  );
  $form['sugarondrupal_rela']['sugarondrupal_relation_fields'] = array(
    '#type'           => 'markup',
    '#value'          => theme('table', $header, $rows),
  );

  return $form;
}

